#!/usr/bin/env python3

"""
This mirror status checker determines whether CSC mirror is up-to-date with upstream
"""

import time
import sys
import requests

from projects import *
import json


def safe_print(*args, **kwargs):
    # When run with 'chronic' and 'timeout', stdout gets suppressed
    # due to buffering. Make sure to always flush the output.
    print(*args, **kwargs, flush=True)


if __name__ == "__main__":

    exit_code = 0

    data_file = 'data.json'
    if len(sys.argv) > 1:
        data_file = sys.argv[1]
    data = json.load(open(data_file))

    current_time = int(time.time())
    for project in data:
        try:
            project_class = getattr(sys.modules[__name__], project)

            # Skip projects we no longer mirror
            if data[project].get('exclude', False):
                continue
            checker_result = project_class.check(data, project, current_time)
            if project in ["CPAN", "ubuntu", "ubuntu_releases", "manjaro", "mxlinux", "cran", "ctan", "gentooportage"]:
                if checker_result:
                    safe_print(f"Success: {project} up-to-date")
                else:
                    safe_print(f"Failure: {project} out-of-sync")

                    # Exit with non-zero status if any of the projects are not up-to-date
                    exit_code = 1
                continue
            if checker_result:
                data[project]["out_of_sync_since"] = None
            elif data[project]["out_of_sync_since"] is None:
                data[project]["out_of_sync_since"] = current_time
            elif current_time - data[project]["out_of_sync_since"] \
                    > data[project]["out_of_sync_interval"]:
                safe_print(f"Failure: {project} out-of-sync")

                # Exit with non-zero status if any of the projects are not up-to-date
                exit_code = 1
                continue
            safe_print(f"Success: {project} up-to-date")
        except requests.exceptions.RequestException as err:
            safe_print(f"Error: {project}\n{err}")
    with open(data_file, "w", encoding="utf-8") as file:
        json.dump(data, file, indent='\t')

    sys.exit(exit_code)
